/*
 * Copyright 2007 Ask Hjorth Larsen
 *
 * asklarsen@gmail.com
 *
 * This file is part of Wires.
 * 
 * Wires is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wires is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Wires.  If not, see <http://www.gnu.org/licenses/>.
 */

package jwires;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author ask
 */
public class Board extends JPanel {
    
    private final TileMap map;
    private int tileSize = 60;
    public final static Color lockedColor = new Color(222, 222, 222);
    public final static Color monoValentColor = new Color(0, 170, 140);
    public final static Color filledColor = new Color(0, 110, 100);
    public final static Color wireColor = Color.BLACK;
    
    /** Creates a new instance of Board */
    public Board(TileMap map) {
        this.map = map;
        setBackground(Color.white);
        Listener listener = new Listener();
        addMouseListener(listener);
        addMouseWheelListener(listener);
        setPreferredSize(new Dimension(tileSize*map.width, tileSize*map.height));
    }
    
    protected void paintComponent(Graphics g){
        super.paintComponent(g);
        // replace with iteration over neighbours or something
        // ... or maybe that is not necessary.
        for(int i = 0; i < map.width; i++){
            for(int j = 0; j < map.height; j++){
                drawTile(map.getTile(i, j), g, i*tileSize, j*tileSize);
            }
        }
    }
    
    public void drawTile(Tile tile, Graphics g, int x, int y){
        if(tile.isLocked()){
            g.setColor(lockedColor);
            g.fillRect(x, y, tileSize, tileSize);
        }
        
        int chunk = tileSize/5;
        if(tile.getConnectionCount() == 1){
            g.setColor(monoValentColor);
            g.fillRect(x+chunk, y+chunk, 3*chunk, 3*chunk);
        }
        
        g.setColor(tile.isMarked() ? filledColor : wireColor);
        if(tile.isConnected(0)){
            g.fillRect(x+2*chunk, y+2*chunk, 3*chunk, 1*chunk);
        }
        if(tile.isConnected(1)){
            g.fillRect(x+2*chunk, y+0*chunk, 1*chunk, 3*chunk);
        }
        if(tile.isConnected(2)){
            g.fillRect(x+0*chunk, y+2*chunk, 3*chunk, 1*chunk);
        }
        if(tile.isConnected(3)){
            g.fillRect(x+2*chunk, y+2*chunk, 1*chunk, 3*chunk);
        }
        
        /*if(tile.isMarked()){
            int d = chunk/3;
            g.setColor(Color.red);
            if(tile.isConnected(0)){
                g.fillRect(x + 2*chunk + d, y + 2*chunk + d, 3*chunk - d, d);
            }
            if(tile.isConnected(1)){
                g.fillRect(x + 2*chunk + d, y, d, 3*chunk - d);
            }
            if(tile.isConnected(2)){
                g.fillRect(x, y + 2*chunk + d, 3*chunk - d, d);
            }
            if(tile.isConnected(3)){
                g.fillRect(x + 2*chunk + d, y + 2*chunk + d, d, 3*chunk - d);
            }
        }*/
    }
    
    private boolean floodFilled = false;
    
    private class Listener implements MouseListener, MouseWheelListener {
        public void mouseClicked(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
            int button = e.getButton();
            int x = map.width * e.getX()/(tileSize*map.width);
            int y = map.height * e.getY()/(tileSize*map.height);
            if(button == e.BUTTON1){
                map.getTile(x, y).toggleLock();
                repaint();            
            } else if(button == e.BUTTON3){
                if(floodFilled){
                    map.clearMarks();
                    floodFilled = false;
                } else {
                    floodFilled = true;
                    int connectedCount = map.floodFill(x, y);
                    if(connectedCount == map.width * map.height){
                        // All tiles connected. We have a winner.
                        for(Tile tile : map.getTiles()){
                            tile.setLocked(true);
                        }
                        Board.this.repaint();
                        JOptionPane.showMessageDialog(null, "Congratulations!", "Game over", 
                                JOptionPane.WARNING_MESSAGE);
                    }
                }
                repaint();
            }
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseWheelMoved(MouseWheelEvent e) {
            int rotation = e.getWheelRotation();
            int x = map.width * e.getX()/(tileSize*map.width);
            int y = map.height * e.getY()/(tileSize*map.height);
            Tile tile = map.getTile(x, y);
            if(!tile.isLocked()){
                map.getTile(x,y).rotate(rotation);
            }
            repaint();
        }
        
    }
    
}
